************************************************************
* Conway's Life - a 16 x 16 version for the Dragon text display.  
* Inspired by Mastering Machine Code on your ZX Spectrum, Toni Baker
* 1983, p217
************************************************************
*-----------------------------------------------------------
* Set up constants
* ----------------------------------------------------------
	org	$4000
table	fcb	-17,-16,-15,1,17,16,15,-1
        fcb	0		;end of table marker
	rmb	7		;space for generation counter display
dump    equ	$40B0		;working area to calulate next generation
dstart	equ	$400		;display start
dsize	equ	$200		;total display size
dend    equ 	dstart+dsize 	;end of display
rwidth	equ	16		;row width in column bytes
thresh	equ	60		;test threshold to decide which char to print
char1	equ	79		;letter 'O' = live cell
char2	equ	143		;blank/empty = empty cell
seed 	equ	$978E		;rom address of random number generator
rand	equ	$116		;rom address of random number
timer	equ	$112		;rom address of timer         

*------------------------------------------------------
* MAIN - start of main program
*------------------------------------------------------
	ldx	#dstart
	ldb	#rwidth
*------------------------------------------------------
* GENONE - create first generation
*------------------------------------------------------

genone	pshs	x,y,b		;Generate random number
        jsr	seed		;and store in A
        puls	x,y,b     
	lda	rand
*decide whether to print alive or empty cell character
	cmpa	#thresh		;Is random value above or below threshold value?
        ble	empty		;If less than then empty cell
        lda	#char1		;otherwise load live cell character
        bra	live		
empty	lda	#char2		;If empty cell, display blank
live	sta	,x+		;Display the cell
        decb			;Count down the display columns
        bne 	genone		;and repeat for each cell until 16
        ldb	#rwidth		;Load row width of 16
        abx			;Add it to end of row just done to force onto next row
        cmpx	#dend		;Has the end of the display area been reeached?
        blo	genone		;Keep printing rows until all done
          
*---------------------------------------------------------
* COPY TO DUMP - copy current generation to working space
*--------------------------------------------------------
cpy2dp	leax	dstart,pcr
        leay	dump,pcr
          
        ldb	#rwidth
copy    lda	,x+
        sta	,y+
        decb
        bne	copy
        ldb	#rwidth
        abx
        cmpx	#dend 
        blo	copy      	;copy from display to dump until done

*----------------------------------------------------------
* NEIGHBOURS - calculate neighbours
*----------------------------------------------------------
*                    
nxtgen  leax	dstart,pcr
        leay	dump,pcr
        clra
	ldb	#rwidth   	;load row width 
        pshs b         		;store it so can reuse B
nxtcell	leau	table,pcr
	clrb           		;Use B to count cell neighbours    
nxtdis	lda 	,u+       	;Load first cell offset into A
        cmpa	#0        	;End of table - only shows 00 in register on 9th
	beq	counted   	;Finish counting neigbours when reach end of table
	lda	a,y       	;otherwise point to neighbour (in dump of display)
	cmpa	#char1    	;Is neighbour a live cell?
        bne	nxtdis    	;If not, check next neighbour
        incb	          	;otherwise add to count of live neighbour cells
        bra  	nxtdis 		;then check next neighbour
counted cmpb	#2        	;If there are less than two neigbouring live cells
	blt	nocell    	;then current cell dies
        cmpb	#4        	;If there are four or more neighbouring live cells
        bge	nocell    	;then current cell dies
        cmpb	#3        	;If there are three cells
        beq	cell      	;then cell lives
	lda	,y	  	;Catchall - if none of the above rules apply, maintain 
				;what is in the present cell position in the dump
        bra	put	  	;Display whatever is there
cell	lda	#char1    	;If cell lives load live cell character
	bra 	put       	;and go to display it
nocell	lda	#char2    	;or else load empty cell character
put     sta	,x+       	;Display cell character (live or empty) and move to next 
				;display position
	leay	1,y       	;Move to next cell position in dump
        
*----------------------------------------------------------
* Check whether row width has reached 16 characters          
*----------------------------------------------------------
         puls b         	;Retrieve row width    
         decb           	;Reduce it by 1 
         pshs b         	;Store it again
	 bne  nxtcell   	;Has it reached zero? No, then check whether or not we have 
	 			;printed last cell position
         ldb	#rwidth 	;When row complete, load B with maximum row width 
	 			;i.e. 16 chars
         pshs b         	;Store it ready for next cell loop
         abx			;Add row width total to 16th display cell in row to force 
	 			;move to next row   
	 cmpx #dend     	;Has the end of the display been reached?
         blo  nxtcell  

*--------------------------------------------------------
* Delay loop
*--------------------------------------------------------
deloop    ldy  #8000
delay     leay -1,y
          bne  delay

*--------------------------------------------------------
* NEXTGEN - Do the next generation
*--------------------------------------------------------
	  bra cpy2dp    ;When all cells displayed, copy current displayed gen to dump
          rts

